﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    /**
     * 余额支付
     */
    public class AcctPaymentDemo
    {
        
        /**
         * 余额支付类交易
         */
        public static void AcctPaymentTrans()
        {
            // 初始化商户配置
            InitMerConfig.init();
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            // 余额支付请求流水号
            string reqSeqId = nowStr + "01234567890";
            // 余额支付
            AcctPayment(nowDate, reqSeqId);
            // 余额支付查询
            AcctPaymentQuery(nowDate, reqSeqId);
            // 余额支付退款
            // 余额支付退款请求流水号
            string refReqSeqId = nowStr + "01234567890";
            AcctPaymentRefund(nowDate, refReqSeqId, nowDate, reqSeqId);
            // 余额支付退款查询
            AcctPaymentRefundQuery(nowDate, refReqSeqId);
        }

        /**
         * 余额支付
         */
        public static void AcctPayment(string nowDate, string reqSeqId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号(入款)
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 出款商户号
            transParam.Add("out_huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 订单金额
            transParam.Add("ord_amt", "0.02");
            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("device_type", "1");
            deviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("device_info", deviceInfo);
            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("transferType", "余额支付");            
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));
            // 分账串
            // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            transParam.Add("acct_payment_split_bunch", acctSplitBunch);
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topat.acctPayment" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topat.acctPayment", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 余额支付查询
         */
        public static void AcctPaymentQuery(string nowDate, string reqSeqId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 原交易请求流水号
            transParam.Add("org_req_seq_id", reqSeqId);
            // 原交易请求日期
            transParam.Add("org_req_date", nowDate);
            // 分配商户号(入款)
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topat.acctPaymentQuery" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topat.acctPaymentQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 余额支付退款
         */
        public static void AcctPaymentRefund(string nowDate, string reqSeqId, string orgReqDate, string orgReqSeqId)
        {
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 余额支付退款请求流水号
            dataParams.Add("req_seq_id", reqSeqId);
            // 余额支付请求日期
            dataParams.Add("req_date", nowDate);
            // 商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 原交易请求日期
            dataParams.Add("org_req_date", orgReqDate);
            // 原交易请求流水号
            dataParams.Add("org_req_seq_id", orgReqSeqId);
            // 退款金额
            dataParams.Add("ord_amt", "0.02");
            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("transferType", "余额支付退款");            
            // 注意，风控信息为JSON字符串
            dataParams.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topat.acctPaymentRefund", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topat.acctPaymentRefund", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }
            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 余额支付退款查询
         */
        public static void AcctPaymentRefundQuery(string nowDate, string reqSeqId)
        {
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 退款请求流水号
            dataParams.Add("req_seq_id", reqSeqId);
            // 退款请求日期
            dataParams.Add("req_date", nowDate);
            // 商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topat.acctPaymentRefundQuery", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topat.acctPaymentRefundQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }
            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }
    }
}