using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    public class BankInstallmentDemo
    {

        public static void BankInstallmentTest()
        {
            // 商户配置初始化
            InitMerConfig.init();

            // 银行卡分期签约
            string signDateStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string signDate = DateTime.Now.ToString("yyyyMMdd");
            string signReqSeqId = signDateStr + "01234567890";
            BankInstallSign(signDate, signReqSeqId);

            // 银行卡分期支付请求
            string payDateStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string payDate = DateTime.Now.ToString("yyyyMMdd");
            string payReqSeqId = payDateStr + "01234567890";
            string payMerOrdId = payDateStr + "01234";
            BankInstallPayment(payDate, payReqSeqId, payMerOrdId);

             // 银行卡分期支付确认
            string confirmDateStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string confirmDate = DateTime.Now.ToString("yyyyMMdd");
            string confirmReqSeqId = confirmDateStr + "01234567890";
            string confirmMerOrdId = confirmDateStr + "01234";
            BankInstallPayConfirm(confirmDate, confirmReqSeqId, confirmMerOrdId, payDate, payReqSeqId);

             // 银行卡分期支付确认
            string hfSeqId = "01234";
            BankInstallQuery(confirmDate, confirmReqSeqId, hfSeqId);

             // 银行卡分期退款
            string refundDateStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string refundDate = DateTime.Now.ToString("yyyyMMdd");
            string refundReqSeqId = refundDateStr + "01234567890";
            string refundMerOrdId = refundDateStr + "01234";
            BankInstallRefund(refundDate, refundReqSeqId, refundMerOrdId, payDate, payReqSeqId, "", "");

        }

        /**
         * 银行卡分期签约
         */
        public static void BankInstallSign(string signDate, string signReqSeqId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", signReqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", signDate);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 银行编号
            transParam.Add("bank_no", "03050000");
            // 持卡人姓名
            transParam.Add("card_name", "张三");
            // 银行卡号
            transParam.Add("card_no", "6226888896666124");
            // 银行卡手机号
            transParam.Add("card_mobile", "17688886666");
            // 证件类型  01：身份证
            transParam.Add("id_type", "01");
            // 证件号
            transParam.Add("id_no", "230404197008080666");
            // CVV2
            transParam.Add("cvv2", "666");
            // 有效期
            transParam.Add("valid_date", "0325");
            // 卡类型 D:借记卡 C:信用卡 P:存折 Y:预付费卡 V:虚拟账户 Z:借贷合一卡
            transParam.Add("card_type", "C");
            // 银行账户类型 E：对公，P：对私
            transParam.Add("card_acct_type", "P");
            // 交易金额 单位元，需保留小数点后两位
            transParam.Add("trans_amt", "100.00");

            // DateTime dt = DateTime.Now;
            // string timeExpire = dt.AddMinutes(2).ToString("yyyyMMddHHmmss");
            //  // 订单有效期,非必填
            // transParam.Add("time_expire", timeExpire);

            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("device_type", "1");
            deviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("terminal_device_info", deviceInfo);

            // 异步通知地址
            transParam.Add("notify_url", "virgo://http://www.vclinker.com");
            // 页面跳转地址(支付成功后，前端跳转地址)
            transParam.Add("business_url", "http://www.baidu.com");

            // 请求类型 0:pc 1:安卓 2：ios
            transParam.Add("request_type", "0");
            // 业务地址 6位地区编码
            transParam.Add("place_id", "200000");

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("分期签约参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("top.bccredit.sign", transParam);               
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("top.bccredit.sign", transParam, "merchantKey2");
                 Console.WriteLine("分期签约结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 银行卡分期支付请求
         */
        public static void BankInstallPayment(string payDate, string payReqSeqId, string payMerOrdId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", payReqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", payDate);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 商户订单号
            transParam.Add("mer_ord_id", payMerOrdId);

            // 银行编号
            transParam.Add("bank_no", "03050000");
            // 持卡人姓名
            transParam.Add("card_name", "张三");
            // 银行卡号
            transParam.Add("card_no", "6226888896666124");
            // 银行卡手机号
            transParam.Add("card_mobile", "17688886666");
            // 证件类型  01：身份证
            transParam.Add("id_type", "01");
            // 证件号
            transParam.Add("id_no", "230404197008080666");
            // CVV2
            transParam.Add("cvv2", "666");
            // 有效期
            transParam.Add("valid_date", "0325");
            // 卡类型 D:借记卡 C:信用卡 P:存折 Y:预付费卡 V:虚拟账户 Z:借贷合一卡
            transParam.Add("card_type", "C");
            // 银行账户类型 E：对公，P：对私
            transParam.Add("card_acct_type", "P");
            // 交易金额 单位元，需保留小数点后两位
            transParam.Add("trans_amt", "100.00");

            // 手续费收取方式 1-实收 2-后收
            transParam.Add("fee_rec_type", "2");

            // DateTime dt = DateTime.Now;
            // string timeExpire = dt.AddMinutes(2).ToString("yyyyMMddHHmmss");
            //  // 订单有效期,非必填
            // transParam.Add("time_expire", timeExpire);

            // 网联扩展字段
            Dictionary<string, Object> extendInfo = new Dictionary<string, Object>();
            extendInfo.Add("pyee_acct_id", "6214***************6688");// 收款方账户编号
            extendInfo.Add("pyee_nm", "李四");// 收款方账户名称
            extendInfo.Add("pyee_acct_tp", "04");// 收款方账户类型 （见数据字典的收款方账户类型, 默认值：04 单位支付账户）
            extendInfo.Add("mer_type", "02");// 二级商户类型： 01：法人02：其他组织03：个体工商户04：自然人
            extendInfo.Add("mer_id_type", "00");// 二级商户证件类型：00：身份证01：护照02：军官证03：军官证04：回乡证05：户口簿06：警官证07：临时身份证08：台胞证09：港澳通行证10：企业营业执照11：法人代表证12：外国人永久居留证，即绿卡13：其他
            extendInfo.Add("mer_id_no", "310528198006088686");// 二级商户证件编码
            extendInfo.Add("mer_cat_code", "8541");// 二级商户行业类别 参照银联MCC编码
            extendInfo.Add("goods_short_name", "*********");//
            extendInfo.Add("biz_tp", "100001");//
            transParam.Add("extend_pay", extendInfo);

            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("device_type", "1");
            deviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("terminal_device_info", deviceInfo);

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("longitude", ""); // 经度
            riskCheckInfo.Add("latitude", ""); // 纬度
            riskCheckInfo.Add("base_station", "");    // 基站地址        
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));


            // 异步通知地址
            transParam.Add("notify_url", "virgo://http://www.vclinker.com");


            // 分期期数 3 6 12 24
            transParam.Add("instalments_num", "3");
            // 延时标记 1为延迟 0为不延迟
            transParam.Add("is_delay_acct", "0");

            // // 分账串
            // // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            // Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            // List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // // 分账方1
            // Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            // acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // acctInfo1.Add("div_amt" , "0.01");
            // acctInfos.Add(acctInfo1);
            // // 分账方2
            // Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            // acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            // acctInfo2.Add("div_amt" , "0.01");
            // acctInfos.Add(acctInfo2);
            // acctSplitBunch.Add("acct_infos", acctInfos);
            // transParam.Add("acct_split_bunch", acctSplitBunch);


            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("分期支付参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("top.bccredit.paymentApply", transParam);
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("top.bccredit.paymentApply", transParam, "merchantKey2");
                 Console.WriteLine("分期支付结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }


        /**
         * 银行卡分期支付确认
         */
        public static void BankInstallPayConfirm(string confirmDate, string confirmReqSeqId, string confirmMerOrdId, string oldDate, string oldReqSeqId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", confirmReqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", confirmDate);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 商户订单号
            transParam.Add("mer_ord_id", confirmMerOrdId);
             // 原请求流水号
            transParam.Add("org_seq_id", oldReqSeqId);
             // 原请求时间
            transParam.Add("org_seq_date", oldDate);

           
            // CVV2
            transParam.Add("cvv2", "666");
            // 有效期
            transParam.Add("valid_date", "0325");
            // 手机验证码
            transParam.Add("verify_code", "123456");
            

            // DateTime dt = DateTime.Now;
            // string timeExpire = dt.AddMinutes(2).ToString("yyyyMMddHHmmss");
            //  // 订单有效期,非必填
            // transParam.Add("time_expire", timeExpire);

            // 网联扩展字段
            Dictionary<string, Object> extendInfo = new Dictionary<string, Object>();
            extendInfo.Add("pyee_acct_id", "6214***************6688");// 收款方账户编号
            extendInfo.Add("pyee_nm", "李四");// 收款方账户名称
            extendInfo.Add("pyee_acct_tp", "04");// 收款方账户类型 （见数据字典的收款方账户类型, 默认值：04 单位支付账户）
            extendInfo.Add("mer_type", "02");// 二级商户类型： 01：法人02：其他组织03：个体工商户04：自然人
            extendInfo.Add("mer_id_type", "00");// 二级商户证件类型：00：身份证01：护照02：军官证03：军官证04：回乡证05：户口簿06：警官证07：临时身份证08：台胞证09：港澳通行证10：企业营业执照11：法人代表证12：外国人永久居留证，即绿卡13：其他
            extendInfo.Add("mer_id_no", "310528198006088686");// 二级商户证件编码
            extendInfo.Add("mer_cat_code", "8541");// 二级商户行业类别 参照银联MCC编码
            extendInfo.Add("goods_short_name", "*********");//
            extendInfo.Add("biz_tp", "100001");//
            transParam.Add("extend_pay", extendInfo);

            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("device_type", "1");
            deviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("device_info", deviceInfo);

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("longitude", ""); // 经度
            riskCheckInfo.Add("latitude", ""); // 纬度
            riskCheckInfo.Add("base_station", "");    // 基站地址        
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));


            // 异步通知地址
            transParam.Add("notify_url", "virgo://http://www.vclinker.com");


            

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("分期确认参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("top.bccredit.paymentApply", transParam);
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("top.bccredit.paymentApply", transParam, "merchantKey2");
                 Console.WriteLine("分期确认结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 银行卡分期交易查询
         */
        public static void BankInstallQuery(string confirmDate, string confirmReqSeqId, string hfSeqId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);


            // 原始请求日期
            transParam.Add("req_date", confirmDate);
            // 原始请求流水号 只能上送银行卡分期确认交易的流水号req_seq_id和hf_seq_id二选一
            transParam.Add("req_seq_id", confirmReqSeqId);
            // // 汇付流水号 只能上送银行卡分期确认交易的流水号req_seq_id和hf_seq_id二选一
            // transParam.Add("hf_seq_id", hfSeqId);
            

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("分期查询参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("topqur.bcCreditPayQuery", transParam);
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("topqur.bcCreditPayQuery", transParam, "merchantKey2");
                 Console.WriteLine("分期查询结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 银行卡分期退款
         */
        public static void BankInstallRefund(string refundDate, string refundReqSeqId, string refundMerOrdId, string oldDate, string oldReqSeqId, string oldPartOrdId, string oldHfSqlId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);


            // 请求时间
            transParam.Add("req_date", refundDate);
            // 请求流水号
            transParam.Add("req_seq_id", refundReqSeqId);
            // 商户订单号
            transParam.Add("mer_ord_id", refundMerOrdId);
            // 退款金额 单位元，需保留小数点后两位，例如： 1.00
            transParam.Add("ord_amt", "100.00");
            // 原交易请求流水号 只能上送银行卡分期支付请求的流水号req_seq_id、hf_seq_id和party_order_id三选一
            transParam.Add("org_req_seq_id", oldReqSeqId);
            // // 原交易支付凭证号 只能上送银行卡分期支付请求的流水号req_seq_id、hf_seq_id和party_order_id三选一
            // transParam.Add("party_order_id", oldPartOrdId);
            // // 原交易全局流水号 只能上送银行卡分期支付请求的流水号req_seq_id、hf_seq_id和party_order_id三选一
            // transParam.Add("hf_seq_id", oldHfSqlId);
            // 原交易请求日期
            transParam.Add("org_req_date", oldDate);
            // 备注 退货原因，超出长度会导致退货失败，UrlEncode进行编码
            transParam.Add("mark", "");
            
            // 异步通知地址
            transParam.Add("notify_url", "virgo://http://www.vclinker.com");


            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("device_type", "1");
            deviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("device_info", deviceInfo);

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("longitude", ""); // 经度
            riskCheckInfo.Add("latitude", ""); // 纬度
            riskCheckInfo.Add("base_station", "");    // 基站地址        
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));

            // // 分账串
            // // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            // Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            // List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // // 分账方1
            // Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            // acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // acctInfo1.Add("div_amt" , "0.01");
            // acctInfos.Add(acctInfo1);
            // // 分账方2
            // Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            // acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            // acctInfo2.Add("div_amt" , "0.01");
            // acctInfos.Add(acctInfo2);
            // acctSplitBunch.Add("acct_infos", acctInfos);
            // transParam.Add("acct_split_bunch", acctSplitBunch);


            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("分期退款参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("topqur.bcCreditPayQuery", transParam);
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("topqur.bcCreditPayQuery", transParam, "merchantKey2");
                 Console.WriteLine("分期退款结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

    }
}