﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    public class TradeConfirmDemo
    {
        public static void TradeConfirm()
        {
            // 初始化商户配置
            InitMerConfig.init();
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");    
            // 交易确认请求日期
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            // 交易确认请求流水号
            string reqSeqId = nowStr + "01234567890";
            // 交易确认
            TradeConfirm(nowDate, reqSeqId);
            // 交易确认查询
            TradeConfirmQuery(nowDate, reqSeqId);
            // 交易确认退款
            string refNowStr = DateTime.Now.ToString("yyyyMMddHHmmss"); 
            // 退款请求流水号
            string refReqSeqId = refNowStr + "01234567890";
            TradeConfirmRefund(nowDate, refReqSeqId, nowDate, reqSeqId);
            // 交易确认列表查询
            PayConfirmListQuery();
        }

        /**
         * 交易确认
         */
        public static void TradeConfirm(string nowDate, string reqSeqId)
        {
            // merOrdId = nowStr + "1234567890";
            string merOrdId = reqSeqId;
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户订单号
            transParam.Add("mer_ord_id", merOrdId);
			// 原交易请求流水号
            transParam.Add("org_req_seq_id", "2021042214110401234567890");
            // 原交易请求日期
			transParam.Add("org_req_date", "20210422");
            // 分账串
            // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            transParam.Add("acct_split_bunch", acctSplitBunch);
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topat.payConfirm" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topat.payConfirm", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 交易确认查询
         */
        public static void TradeConfirmQuery(string nowDate, string reqSeqId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 原交易请求流水号
            transParam.Add("req_seq_id", reqSeqId);
            // 原交易请求日期
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topqur.payConfirmQuery" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topqur.payConfirmQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 交易确认退款
         */
        public static void TradeConfirmRefund(string nowDate, string reqSeqId, string orgReqDate, string orgReqSeqId)
        {
            // merOrdId = nowStr + "1234567890";
            string merOrdId = reqSeqId;
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号(入款)
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 出款商户号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户订单号
            transParam.Add("mer_ord_id", merOrdId);
            // 原交易确认请求流水号
            transParam.Add("org_req_seq_id", orgReqSeqId);
            // 原交易确认请求日期
            transParam.Add("org_req_date", orgReqDate);
            // 分账串
            // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            transParam.Add("acct_split_bunch", acctSplitBunch);
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("toprrc.payConfirmRefund" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("toprrc.payConfirmRefund", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 交易确认列表查询
         */
        public static void PayConfirmListQuery()
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 原交易请求流水号(非交易确认请求流水号)
            transParam.Add("trans_req_seq_id", "2021042214110401234567890");
            // 原交易请求日期(非交易确认请求日期)
            transParam.Add("trans_req_date", "20210422");
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("topqur.payConfirmListQuery" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topqur.payConfirmListQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }
    }
}