using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    public class TransCashDemo
    {

        public static void TransCashTest()
        {
            // 商户配置初始化
            InitMerConfig.init();

            string transDateStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string transDate = DateTime.Now.ToString("yyyyMMdd");
            // 取现交易请求流水号
            string transReqSeqId = transDateStr + "01234567890";
            string transMerOrdId = transDateStr + "01234";

            // 取现
            Enchashment(transDate, transReqSeqId, transMerOrdId);

            string queryDateStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string queryDate = DateTime.Now.ToString("yyyyMMdd");
            // 取现交易查询请求流水号
            string queryReqSeqId = queryDateStr + "01234567890";
            // 取现查询
            EnchashmentQuery(queryDate, queryReqSeqId, transDate, transReqSeqId, transMerOrdId);

        }

        /**
         * 取现交易
         */
        public static void Enchashment(string transDate, string transReqSeqId, string transMerOrdId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", transReqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", transDate);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 商户订单号
            transParam.Add("mer_ord_id", transMerOrdId);
            // 取现卡序列号
            transParam.Add("token_no", DemoConstants.TOKEN_NO);
            // 取现金额.单位:元
            transParam.Add("cash_amt", "0.01");
            // 入账类型
            // T0：当日到账；T1：T1次工作日到账；D1：次自然日到账；
            transParam.Add("into_acct_date_type", "T1");
            // 异步通知地址
            transParam.Add("notify_url", "virgo://http://www.vclinker.com");

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("longitude", ""); // 经度
            riskCheckInfo.Add("latitude", ""); // 纬度
            riskCheckInfo.Add("base_station", "");    // 基站地址        
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("取现参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("toppas.enchashment", transParam);
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("toppas.enchashment", transParam, "merchantKey2");
                Console.WriteLine("取现结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 取现交易查询
         */
        public static void EnchashmentQuery(string queryDate, string queryReqSeqId, string oldDate, string oldReqSeqId, string oldMerOrdId)
        {
            // 组装参数
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", queryReqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", queryDate);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 原交易请求日期
            transParam.Add("trans_req_date", oldDate);
            // 原交易的商户订单号 trans_mer_ord_id和hf_seq_id二选一
            transParam.Add("trans_mer_ord_id", oldMerOrdId);
            // 原交易的汇付流水号 trans_mer_ord_id和hf_seq_id二选一
            // transParam.Add("hf_seq_id", oldReqSeqId);
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try
            {
                Console.WriteLine("取现查询参数：" + JsonConvert.SerializeObject(transParam, Formatting.None));
                result = BasePayClient.postRequest("topqur.cashQuery", transParam);
                // 使用指定配置调用接口
                // result = BasePayClient.postRequest("topqur.cashQuery", transParam, "merchantKey2");
                Console.WriteLine("取现查询结果：" + JsonConvert.SerializeObject(result, Formatting.None));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 调用成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

    }
}