using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{

    // 聚合扫码实例
    public class TransDemo
    {
        public static void TransTest(){
            // 商户配置初始化
            InitMerConfig.init();

            // 聚合正扫
            Dictionary<string, Object> pullPayInfo = PullPayInfo();

            // 聚合反扫
            Dictionary<string, Object> authCodePayResult = AuthCodePay();

            // 交易查询
            QueryTrans(pullPayInfo);

            // 退款
            Dictionary<string, Object> refundResult = RefundTrans(authCodePayResult);

            // 退款查询
            QueryRefund(refundResult);

            // 关单
            // 暂停1分钟后再执行关单
            // System.Threading.Thread.Sleep(60000);
            CloseTrans(pullPayInfo);

            // 获取银联用户标识
            GetUnionUserId();

        }

        // 聚合正扫实例
        public static Dictionary<string, Object> PullPayInfo(){

            DateTime dt = DateTime.Now;
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string reqSeqId = nowStr + "01234567890";
            // merOrdId = nowStr + "1234567890";
            string merOrdId = reqSeqId;
            string timeExpire = dt.AddMinutes(2).ToString("yyyyMMddHHmmss");

            /**
            * 聚合正扫调用实例
            **/
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 系统号，由汇付分配
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户订单号，需保证商户下唯一，推荐采用日期时间+几位流水号的形式，可与req_seq_id相同
            transParam.Add("mer_ord_id", merOrdId);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 交易类型
            // 微信公众号-T_JSAPI 小程序-T_MINIAPP 支付宝JS-A_JSAPI 支付宝正扫-A_NATIVE 银联正扫-U_NATIVE 银联JS-U_JSAPI
            transParam.Add("trade_type" , "A_NATIVE");
            // 渠道号，若需使用自有渠道请联系业务对接人
            transParam.Add("bank_channel_no", "");
            // 交易金额
            transParam.Add("trans_amt", "0.02");
            // 商品描述
            transParam.Add("goods_desc", "测试商品");
            // 设备信息
            Dictionary<string, Object> terminalDeviceInfo = new Dictionary<string, Object>();
            terminalDeviceInfo.Add("device_type", "4");
            terminalDeviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("terminal_device_info", terminalDeviceInfo);

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("ipAddr", "139.207.19.246");            
            Dictionary<string, Object> riskMngInfo = new Dictionary<string, Object>();
            riskMngInfo.Add("subTradeType" , "4300");
            riskCheckInfo.Add("riskMngInfo", riskMngInfo);
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));

            // 异步通知地址
            transParam.Add("notify_url" , "virgo://http://www.vclinker.com");

            // 交易有效期
            transParam.Add("time_expire" , timeExpire);
            // 指定支付方式 是否禁止用户使用信用卡支付。默认不禁用，若禁止请填1
            transParam.Add("limit_pay" , "1");
            // 延时标志 1为延迟 0为不延迟
            transParam.Add("is_delay_acct" , "0");
            // 传入分帐遇到优惠的处理规则 1-按比例分 2-按顺序保障 3-只给交易商户
            transParam.Add("term_div_coupon_type" , "3");
            // 分账串
            // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            transParam.Add("acct_split_bunch", acctSplitBunch);

            // 商户私有域
            transParam.Add("mer_priv", "");
            // 微信扩展参数集合
            transParam.Add("wx_data" , "");
            // 支付宝扩展参数集合
            transParam.Add("alipay_data" , "");
            // 银联扩展参数集合
            transParam.Add("unionpay_data" , "");
            // 手续费类型 01:标准费率线上,02:标准费率线下,03:非盈利费率,04:缴费费率,05:保险费率,06:行业活动费率,07:校园餐饮费率,08:K12中小幼费率
            // 不送时取业务入驻配置的默认费率
            transParam.Add("pay_scene" , "02");
            // 分期贴息类型，1:商户贴息，默认商户不贴息。注意：当此处填1时，花呗商户承担比例hb_fq_seller_percent只能填0
            transParam.Add("fq_mer_discount_flag" , 0);
            // 交易备注
            transParam.Add("remark" , "");

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("top.trans.pullPayInfo" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top.trans.pullPayInfo", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode) || string.Equals("00000100", (string)subRespCode)) 
            {
                // 处理成功或者处理中
                // 获取聚合正扫返回的支付信息 Native支付时使用此二维码信息
                object QrCode;
                if (result.TryGetValue("qr_code", out QrCode)) {
                    Console.WriteLine("支付二维码信息:" + QrCode);
                }
                // js支付信息，js支付及微信公众号支付使用此信息
                object PayInfo;
                if (result.TryGetValue("pay_info", out PayInfo)) {
                    Console.WriteLine("支付二维码信息:" + PayInfo);
                }
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }

            return result;
        }

        // 聚合反扫实例
        public static Dictionary<string, Object> AuthCodePay(){

            DateTime dt = DateTime.Now;
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string reqSeqId = nowStr + "01234567890";
            // merOrdId = nowStr + "1234567890";
            string merOrdId = reqSeqId;
            string timeExpire = dt.AddMinutes(2).ToString("yyyyMMddHHmmss");

            /**
            * 聚合反扫调用实例
            **/
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 系统号，由汇付分配
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户订单号，需保证商户下唯一，推荐采用日期时间+几位流水号的形式，可与req_seq_id相同
            transParam.Add("mer_ord_id", merOrdId);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 渠道号，若需使用自有渠道请联系业务对接人
            transParam.Add("bank_channel_no", "");
            // 交易金额
            transParam.Add("trans_amt", "0.02");
            // 商品描述
            transParam.Add("goods_desc", "测试商品");
            transParam.Add("auth_code" , DemoConstants.WX_AUTH_CODE);
            // 设备信息
            Dictionary<string, Object> terminalDeviceInfo = new Dictionary<string, Object>();
            terminalDeviceInfo.Add("device_type", "4");
            terminalDeviceInfo.Add("device_ip", "139.207.19.246");
            transParam.Add("terminal_device_info", terminalDeviceInfo);

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("ipAddr", "139.207.19.246");            
            Dictionary<string, Object> riskMngInfo = new Dictionary<string, Object>();
            riskMngInfo.Add("subTradeType" , "4300");
            riskCheckInfo.Add("riskMngInfo", riskMngInfo);
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));

            // 异步通知地址
            transParam.Add("notify_url" , "virgo://http://www.vclinker.com");

            // 交易有效期
            transParam.Add("time_expire" , timeExpire);
            // 指定支付方式 是否禁止用户使用信用卡支付。默认不禁用，若禁止请填1
            transParam.Add("limit_pay" , "1");
            // 延时标志 1为延迟 0为不延迟
            transParam.Add("is_delay_acct" , "0");
            // 传入分帐遇到优惠的处理规则 1-按比例分 2-按顺序保障 3-只给交易商户
            transParam.Add("term_div_coupon_type" , "3");
            // 分账串
            // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            transParam.Add("acct_split_bunch", acctSplitBunch);

            // 商户私有域
            transParam.Add("mer_priv", "");
            // 微信扩展参数集合
            transParam.Add("wx_data" , "");
            // 支付宝扩展参数集合
            transParam.Add("alipay_data" , "");
            // 银联扩展参数集合
            transParam.Add("unionpay_data" , "");
            // 手续费类型 01:标准费率线上,02:标准费率线下,03:非盈利费率,04:缴费费率,05:保险费率,06:行业活动费率,07:校园餐饮费率,08:K12中小幼费率
            // 不送时取业务入驻配置的默认费率
            transParam.Add("pay_scene" , "02");

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("top.trans.authCodePay" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top.trans.authCodePay", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode)) 
            {
                // 处理成功
                // 获取交易状态
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
                object wxData = null;
                if (result.TryGetValue("wx_response", out wxData)) {
                    Console.WriteLine(wxData);
                    JObject wxResponse = (JObject)wxData;
                
                    Console.WriteLine("wx_response:" + JsonConvert.SerializeObject(wxResponse, Formatting.None));
                    JToken openid = null;
                    if (wxResponse.TryGetValue("openid", out openid)) {
                        Console.WriteLine("openid:=" + openid);
                    }
                }
            } 
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
                // 后续可调用交易查询获取处理中状态的最终支付结果
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
            return result;
        }

        // 交易查询
        public static Dictionary<string, Object> QueryTrans(Dictionary<string, Object> dict) {
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            dataParams.Add("product_id", DemoConstants.DEMO_PRODUCT_ID);
            dataParams.Add("req_date", dict["req_date"]);
            dataParams.Add("req_seq_id", dict["req_seq_id"]);
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);

            Dictionary<string, Object> result = null;
            // 调用接口,使用默认商户配置时可省略配置key
            try {
                result = BasePayClient.postRequest("topqur.transQuery", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topqur.transQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
            object subRespCode;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode)) 
            {
                // 处理成功
                // 获取交易状态
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);

            } 
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
            return result;
        }

        // 交易退款
        public static Dictionary<string, Object> RefundTrans(Dictionary<string, Object> dict) {
            DateTime dt = DateTime.Now;
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string reqSeqId = nowStr + "01234567890";
            // merOrdId = nowStr + "1234567890";
            string merOrdId = reqSeqId;
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            dataParams.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            dataParams.Add("req_date", nowDate);
            // 商户订单号，需保证商户下唯一，推荐采用日期时间+几位流水号的形式，可与req_seq_id相同
            dataParams.Add("mer_ord_id" , reqSeqId);
            // 系统号，由汇付分配
            dataParams.Add("sys_id" , DemoConstants.DEMO_SYS_ID);
            // 进件接口分配商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 分账串，原路径退回无需上送
            /*
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "0.01");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            dataParams.Add("acct_split_bunch", acctSplitBunch);
            */
            
            // 以下指定原交易的唯一标识，org_req_seq_id、hf_seq_id、party_order_id中3选1
            // 原交易请求流水号
            dataParams.Add("org_req_seq_id", dict["req_seq_id"]);
            // dataParams.Add("org_req_seq_id", "2021051215480201234567890");
            // dataParams.Add("hf_seq_id", "002900TOP2A210506093117P359ac132fcd00000");
            // dataParams.Add("party_order_id", "03242105063840898400756");
            // 原交易请求日期，指原交易的请求日期，当根据原交易流水号退货时必送
            dataParams.Add("org_req_date", dict["req_date"]);
            // 退款金额
            dataParams.Add("ord_amt", "0.02");

            // 设备信息
            Dictionary<string, Object> terminalDeviceInfo = new Dictionary<string, Object>();
            terminalDeviceInfo.Add("device_type", "4");
            terminalDeviceInfo.Add("device_ip", "139.207.19.246");
            dataParams.Add("terminal_device_info", terminalDeviceInfo);

            // 备注
            dataParams.Add("mark", "交易退货");
            // 扩展域
            dataParams.Add("mer_priv", "扩展域");

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("ipAddr", "139.207.19.246");            
            Dictionary<string, Object> riskMngInfo = new Dictionary<string, Object>();
            riskMngInfo.Add("subTradeType" , "4300");
            riskCheckInfo.Add("riskMngInfo", riskMngInfo);
            // 注意，风控信息为JSON字符串
            dataParams.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));

            // 异步通知地址
            dataParams.Add("notify_url" , "virgo://http://www.vclinker.com");

            Dictionary<string, Object> result = null;
            // 调用接口,使用默认商户配置时可省略配置key
            try {
                result = BasePayClient.postRequest("toprrc.refund", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("toprrc.refund", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
            object subRespCode;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode)) 
            {
                // 处理成功
                // 获取交易状态
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
                // 交易状态为处理中后续可调用退款查询获取最终状态
            } 
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
                // 交易状态为处理中后续可调用退款查询获取最终状态
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
            return result;
        }

        // 交易退款查询
        public static Dictionary<string, Object> QueryRefund(Dictionary<string, Object> dict) {
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 退款商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 原始退款请求日期
            dataParams.Add("req_date", dict["req_date"]);
            // 分配系统号
            dataParams.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 以下上送原退款的唯一标识、mer_ord_id、req_seq_id、hf_seq_id、out_ord_id可4选1
            // 原退款请求流水号
            dataParams.Add("req_seq_id", dict["req_seq_id"]);

            Dictionary<string, Object> result = null;
            // 调用接口,使用默认商户配置时可省略配置key
            try {
                result = BasePayClient.postRequest("topqur.refundQuery", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("topqur.refundQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
            object subRespCode;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode)) 
            {
                // 处理成功
                // 获取交易状态
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
                // 交易状态为处理中后续可调用退款查询获取最终状态
            } 
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
                // 交易状态为处理中后续可调用退款查询获取最终状态
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
            return result;
        }

        // 交易关单
        public static Dictionary<string, Object> CloseTrans(Dictionary<string, Object> dict) {
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string reqSeqId = nowStr + "01234567890";    
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 关单请求日期
            dataParams.Add("req_date", nowDate);
            // 关单请求流水号
            dataParams.Add("req_seq_id", reqSeqId);
            // 关单交易商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 分配系统号
            dataParams.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 原交易请求日期
            dataParams.Add("org_req_date", dict["req_date"]);
            // 原交易请求流水号
            dataParams.Add("org_req_seq_id", dict["req_seq_id"]);


            Dictionary<string, Object> result = null;
            // 调用接口,使用默认商户配置时可省略配置key
            try {
                result = BasePayClient.postRequest("toprrc.close", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("toprrc.close", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
            object subRespCode;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode) || string.Equals("00000100", (string)subRespCode)) 
            {
                // 处理成功、处理中
                // 获取关单交易状态
                object transStat = null;
                if (result.TryGetValue("trans_stat", out transStat)) {
                    Console.WriteLine("关单结果：" + transStat);
                    // 获取原交易状态
                    object orgTransStat;
                    if (result.TryGetValue("org_trans_stat", out orgTransStat)) {
                        Console.WriteLine("原交易状态=" + orgTransStat);
                    }
                    // 不论原交易状态如何，建议再次调用查询确定交易最终状态
                }
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }


            return result;
        }

        // 获取银联用户标识
        public static Dictionary<string, Object> GetUnionUserId() {
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string reqSeqId = nowStr + "01234567890";    
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 请求日期
            dataParams.Add("req_date", nowDate);
            // 请求流水号
            dataParams.Add("req_seq_id", reqSeqId);
            // 关单交易商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 分配系统号
            dataParams.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 用户授权码
            dataParams.Add("user_auth_code", "6j2MZY3YShmGFn4NHfQqeA==");
            // APP标识,不送默认CloudPay
            dataParams.Add("app_up_identifier", "CloudPay");


            Dictionary<string, Object> result = null;
            // 调用接口,使用默认商户配置时可省略配置key
            try {
                result = BasePayClient.postRequest("top.trans.getUserMark", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top.trans.getUserMark", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
            }
            object userId;
            if (result.TryGetValue("user_id", out userId)) {
                Console.WriteLine("银联用户开放标识：" + userId);
            } else {
                Console.WriteLine("银联用户开放标识获取失败");
            }

            return result;
        }
    }
}
