﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    public class WalletEnchashmentDemo
    {

        public static void WalletEnchashment()
        {
            // 初始化商户配置
            InitMerConfig.init();  
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");  
            // 钱包取现请求流水号
            string reqSeqId = nowStr + "01234567890";
            // 钱包取现
            Enchashment(nowDate, reqSeqId);
            // 钱包取现查询
            string queryNowStr = DateTime.Now.ToString("yyyyMMddHHmmss");  
            // 钱包取现查询请求流水号
            string queryReqSeqId = queryNowStr + "01234567890";
            EnchashmentQuery(nowDate, queryReqSeqId, nowDate, reqSeqId);
        }

        /**
         * 钱包取现
         */
        public static void Enchashment(string nowDate, string reqSeqId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 产品号
            transParam.Add("product_id", DemoConstants.DEMO_PRODUCT_ID);
            // 系统号
            transParam.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 取现卡序号
            transParam.Add("token_no", "5252255");
            // 取现金额
            transParam.Add("cash_amt", "0.01");
            // 入账类型 D0:当日到账;T1:下个工作日到账;D1:下个自然日到账
            transParam.Add("into_acct_date_type", "D0");
            // 异步通知地址
            transParam.Add("notify_url", "virgo://http://www.xxx.com/getResp");
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("ssproxy.walletEnchashment" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("ssproxy.walletEnchashment", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 钱包取现查询
         */
        public static void EnchashmentQuery(string nowDate, string reqSeqId, string orgReqDate, string orgReqSeqId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 产品号
            transParam.Add("product_id", DemoConstants.DEMO_PRODUCT_ID);
            // 原取现交易请求流水号
            transParam.Add("org_req_seq_id", orgReqSeqId);
            // 原取现交易请求日期
            transParam.Add("org_req_date", orgReqDate);
            // 原取现交易系统号
            transParam.Add("org_sys_id", DemoConstants.DEMO_SYS_ID);
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("ssproxy.walletEnchashmentQuery" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("ssproxy.walletEnchashmentQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }
    }
}