﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    public class WalletFrzBalDemo
    {
        /**
         * 钱包余额冻结/解冻
         */
        public static void WalletFrzBalance()
        {   
            // 初始化商户配置
            InitMerConfig.init();  
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");  
            // 钱包余额冻结请求流水号
            string reqSeqId = nowStr + "01234567890";
            // 钱包余额冻结
            FrzWalletBal(nowDate, reqSeqId);
            // 钱包余额解冻
            string frzNowStr = DateTime.Now.ToString("yyyyMMddHHmmss");  
            // 钱包余额解冻请求流水号
            string frzReqSeqId = frzNowStr + "01234567890";
            UnFrzWalletBal(nowDate, frzReqSeqId, nowDate, reqSeqId);
        }

        /**
         * 钱包余额冻结
         */
        public static void FrzWalletBal(string nowDate, string reqSeqId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 冻结金额
            transParam.Add("trans_amt", "0.01");
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("ssproxy.frzWalletBal" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("ssproxy.frzWalletBal", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 钱包余额解冻
         */
        public static void UnFrzWalletBal(string nowDate, string reqSeqId, string orgReqDate, string orgReqSeqId)
        {
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 解冻明细列表
            Dictionary<string, string> unFrzInfo = new Dictionary<string, string>();
            List<Dictionary<string, string>> unFrzInfos = new List<Dictionary<string, string>>();
            // 原冻结请求流水号
            unFrzInfo.Add("org_req_seq_id", orgReqSeqId);
            // 原冻结请求日期
            unFrzInfo.Add("org_req_date", orgReqDate);
            // 原冻结请求系统号
            unFrzInfo.Add("org_sys_id", DemoConstants.DEMO_SYS_ID);
            unFrzInfos.Add(unFrzInfo);
            transParam.Add("un_frz_infos", unFrzInfos);
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("ssproxy.unFrzWalletBal" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("ssproxy.unFrzWalletBal", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                // 成功
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("transStat：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }
    }
}