﻿using System;
using System.Collections.Generic;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using BasePaySdk;

namespace BasePayDemo
{
    public class WapPayDemo
    {

        public static void OnlineTrans()
        {
            // 初始化商户配置
            InitMerConfig.init();
            string nowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            string nowDate = DateTime.Now.ToString("yyyyMMdd");
            // 手机网页支付请求流水号
            string reqSeqId = nowStr + "01234567890";
            // 手机网页支付
            WapPay(nowDate, reqSeqId);
            // 线上交易查询
            OnlineTransQuery(nowDate, reqSeqId);
            // 线上交易退款
            string refNowStr = DateTime.Now.ToString("yyyyMMddHHmmss");
            // 线上交易退款请求流水号
            string refundReqSeqId = refNowStr + "01234567890";
            OnlineRefund(nowDate, refundReqSeqId, nowDate, reqSeqId);
            // 线上交易退款查询
            OnlineRefundQuery(nowDate, refundReqSeqId);
        }

        /**
         * 手机网页支付
         */
        public static void WapPay(string nowDate, string reqSeqId)
        {
            DateTime dt = DateTime.Now;
            string timeExpire = dt.AddMinutes(2).ToString("yyyyMMddHHmmss");
            
            /**
            * 手机网页支付调用实例
            **/
            Dictionary<string, Object> transParam = new Dictionary<string, Object>();
            // 请求流水号，需保证当天商户下唯一，推荐采用日期时间+几位流水号的形式
            transParam.Add("req_seq_id", reqSeqId);
            // 请求日期，请求接口的日期，因服务器时间可能有差异，允许前后1天
            transParam.Add("req_date", nowDate);
            // 分配商户号
            transParam.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 交易金额
            transParam.Add("trans_amt", "200.00");
            // 银行卡号
            transParam.Add("card_no", "6226**********0124");
            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("trx_device_type", "01");
            deviceInfo.Add("trx_device_ip", "139.207.19.246");
            transParam.Add("device_info", deviceInfo);
            
            // 银联扩展信息
            Dictionary<string, Object> extendPay = new Dictionary<string, Object>();
            extendPay.Add("pyer_trx_trm_tp", "02");
            transParam.Add("extend_pay", extendPay);

            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("ipAddr", "139.207.19.246");            
            Dictionary<string, Object> riskMngInfo = new Dictionary<string, Object>();
            riskMngInfo.Add("subTradeType" , "4300");
            riskCheckInfo.Add("riskMngInfo", riskMngInfo);
            // 注意，风控信息为JSON字符串
            transParam.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));

            // 异步通知地址
            transParam.Add("mer_notice_url" , "virgo://http://www.vclinker.com");
            // 页面回调地址
            transParam.Add("business_url", "virgo://http://www.xxx.com/getResp");

            // 交易有效期
            transParam.Add("time_expire" , timeExpire);
            // 分账串
            // 注意只有真实分账场景才需要传入分账串，请避免传入多个分账方为同一商户的分账串
            Dictionary<string, Object> acctSplitBunch = new Dictionary<string, Object>();
            List<Dictionary<string, Object>> acctInfos = new List<Dictionary<string, Object>>();
            // 分账方1
            Dictionary<string, Object> acctInfo1 = new Dictionary<string, Object>();
            acctInfo1.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            acctInfo1.Add("div_amt" , "100.00");
            acctInfos.Add(acctInfo1);
            // 分账方2
            Dictionary<string, Object> acctInfo2 = new Dictionary<string, Object>();
            acctInfo2.Add("huifu_id", DemoConstants.DIV_HUIFU_ID);
            acctInfo2.Add("div_amt" , "100.00");
            acctInfos.Add(acctInfo2);
            acctSplitBunch.Add("acct_infos", acctInfos);
            transParam.Add("acct_split_bunch", acctSplitBunch);

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("top-online-ser.wapPay" , transParam);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top-online-ser.wapPay", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }

            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object formHtml = null;
                result.TryGetValue("form_html", out formHtml);
                Console.WriteLine("formHtml：" + formHtml);
                object formEncoding = null;
                result.TryGetValue("formEncoding", out formEncoding);
                Console.WriteLine("formEncoding：" + formEncoding);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }
        
        /**
         * 线上交易查询
         */
         static void OnlineTransQuery(string reqDate, string reqSeqId)
        {
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 产品号
            dataParams.Add("product_id", DemoConstants.DEMO_PRODUCT_ID); 
            // 原交易请求日期
            dataParams.Add("req_date", reqDate);
            // 原交易请求流水号
            dataParams.Add("req_seq_id", reqSeqId);
            // 商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("top-online-ser.transQuery", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top-online-ser.transQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }
            
            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode)) 
            {
                // 处理成功
                // 获取交易状态
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            } 
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 线上交易退款
         */
        public static void OnlineRefund(string nowDate, string reqSeqId, 
            string orgReqDate, string orgReqSeqId)
        {
            // merOrdId = nowStr + "1234567890";
            string merOrdId = reqSeqId;
            
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 退款请求流水号
            dataParams.Add("req_seq_id", reqSeqId);
            // 退款请求日期
            dataParams.Add("req_date", nowDate);
            // 商户订单号
            dataParams.Add("mer_ord_id", merOrdId);
            // 系统号
            dataParams.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);
            // 原交易请求日期
            dataParams.Add("org_req_date", orgReqDate);
            // 原交易请求流水号
            dataParams.Add("org_req_seq_id", orgReqSeqId);
            // 退款金额
            dataParams.Add("ord_amt", "200.00");
            // 退款异步通知地址
            dataParams.Add("notify_url", "virgo://http://www.vclinker.com");
            // 设备信息
            Dictionary<string, Object> deviceInfo = new Dictionary<string, Object>();
            deviceInfo.Add("trx_device_type", "04");
            deviceInfo.Add("trx_device_ip", "139.207.19.246");
            dataParams.Add("device_info", deviceInfo);
            // 风控信息
            Dictionary<string, Object> riskCheckInfo = new Dictionary<string, Object>();
            riskCheckInfo.Add("ipAddr", "139.207.19.246");            
            Dictionary<string, Object> riskMngInfo = new Dictionary<string, Object>();
            riskMngInfo.Add("subTradeType" , "4300");
            riskCheckInfo.Add("riskMngInfo", riskMngInfo);
            // 注意，风控信息为JSON字符串
            dataParams.Add("risk_check_info", JsonConvert.SerializeObject(riskCheckInfo, Formatting.None));
            
            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("top-online-ser.refund", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top-online-ser.refund", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }
            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode))
            {
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            }
            else if (string.Equals("00000100", (string)subRespCode))
            {
                // 处理中
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            }
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

        /**
         * 线上退款查询
         */
        static void OnlineRefundQuery(string refundReqDate, string refundReqSeqId)
        { 
            // 参数组装
            Dictionary<string, Object> dataParams = new Dictionary<string, Object>();
            // 系统号
            dataParams.Add("sys_id", DemoConstants.DEMO_SYS_ID);
            // 原退款请求日期
            dataParams.Add("req_date", refundReqDate);
            // 原退款请求流水号
            dataParams.Add("req_seq_id", refundReqSeqId);
            // 商户号
            dataParams.Add("huifu_id", DemoConstants.DEMO_HUIFU_ID);

            // 调用接口,使用默认商户配置时可省略配置key
            Dictionary<string, Object> result = null;
            try {
                result = BasePayClient.postRequest("top-online-ser.refundQuery", dataParams);
                // 使用指定配置调用接口
                // result = Dictionary<string, Object> result = BasePayClient.postRequest("top-online-ser.refundQuery", dataParams, "merchantKey2");
            } catch (Exception ex) {
                Console.WriteLine(ex.Message);
                return;
            }
            
            object subRespCode = null;
            result.TryGetValue("sub_resp_code", out subRespCode);
            if (string.Equals("00000000", (string)subRespCode)) 
            {
                // 处理成功
                // 获取交易状态
                object transStat = null;
                result.TryGetValue("trans_stat", out transStat);
                Console.WriteLine("交易状态：" + transStat);
            } 
            else 
            {
                // 失败
                object subRespDesc = null;
                result.TryGetValue("sub_resp_desc", out subRespDesc);
                Console.WriteLine("sub_resp_desc" + subRespDesc);
            }
        }

    }
}